package radioai

import (
	"flag"
	"io/ioutil"
	"log"
	"net"
	"strings"

	"github.com/coreos/go-etcd/etcd"
)

var (
	etcdMachines = flag.String("etcd-servers", "localhost:7800", "Etcd servers (comma-separated list)")
	etcdCertFile = flag.String("etcd-cert", "", "SSL certificate for etcd client")
	etcdKeyFile  = flag.String("etcd-key", "", "SSL private key for etcd client")
)

func loadFile(path string) string {
	data, err := ioutil.ReadFile(path)
	if err != nil {
		log.Fatal(err)
	}
	return string(data)
}

func resolveAll(input []string) []string {
	result := make([]string, 0)
	for _, hostport := range input {
		host, port, err := net.SplitHostPort(hostport)
		if err != nil {
			log.Fatal("Error parsing etcd server spec '%s': %s", hostport, err)
		}
		addrs, err := net.LookupHost(host)
		if err != nil {
			log.Fatal("Error resolving etcd server spec '%s': %s", hostport, err)
		}
		for _, a := range addrs {
			result = append(result, net.JoinHostPort(a, port))
		}
	}
	return result
}

func NewEtcdClient() *etcd.Client {
	machines := resolveAll(strings.Split(*etcdMachines, ","))
	if len(machines) == 0 {
		log.Fatal("No etcd servers specified!")
	}
	c := etcd.NewClient(machines)
	if *etcdCertFile != "" && *etcdKeyFile != "" {
		c.SetScheme(etcd.HTTPS)
		if _, err := c.SetCertAndKey(loadFile(*etcdCertFile), loadFile(*etcdKeyFile)); err != nil {
			log.Fatal("Error setting up SSL for etcd client: %s", err)
		}
	}
	return c
}
